@@ -7,15 +7,18 @@ import (
 	"context"
 	"encoding/base64"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"log"
-	"net/http"
+	"time"
 
-	"github.com/hashicorp/go-multierror"
+	"github.com/hashicorp/go-azure-helpers/lang/response"
 	"github.com/hashicorp/go-uuid"
+	"github.com/jackofallops/giovanni/storage/2023-11-03/blob/blobs"
+
 	"github.com/hashicorp/terraform/internal/states/remote"
 	"github.com/hashicorp/terraform/internal/states/statemgr"
-	"github.com/tombuildsstuff/giovanni/storage/2018-11-09/blob/blobs"
+	"github.com/hashicorp/terraform/internal/tfdiags"
 )
 
 const (
@@ -24,6 +27,15 @@ const (
 	lockInfoMetaKey = "terraformlockid"
 )
 
+const veryLongTimeout = 9999 * time.Hour
+
+// newCtx creates a context with a (meaningless) deadline.
+// This is only to make the go-azure-sdk/sdk/client Client happy.
+func newCtx() context.Context {
+	ctx, _ := context.WithTimeout(context.TODO(), veryLongTimeout)
+	return ctx
+}
+
 type RemoteClient struct {
 	giovanniBlobClient blobs.Client
 	accountName        string
@@ -33,34 +45,41 @@ type RemoteClient struct {
 	snapshot           bool
 }
 
-func (c *RemoteClient) Get() (*remote.Payload, error) {
+func (c *RemoteClient) Get() (*remote.Payload, tfdiags.Diagnostics) {
+	var diags tfdiags.Diagnostics
 	options := blobs.GetInput{}
 	if c.leaseID != "" {
 		options.LeaseID = &c.leaseID
 	}
 
-	ctx := context.TODO()
-	blob, err := c.giovanniBlobClient.Get(ctx, c.accountName, c.containerName, c.keyName, options)
+	ctx := newCtx()
+	blob, err := c.giovanniBlobClient.Get(ctx, c.containerName, c.keyName, options)
 	if err != nil {
-		if blob.Response.IsHTTPStatus(http.StatusNotFound) {
+		if response.WasNotFound(blob.HttpResponse) {
 			return nil, nil
 		}
-		return nil, err
+		return nil, diags.Append(err)
+	}
+
+	if blob.Contents == nil {
+		return nil, diags
 	}
 
 	payload := &remote.Payload{
-		Data: blob.Contents,
+		Data: *blob.Contents,
 	}
 
 	// If there was no data, then return nil
 	if len(payload.Data) == 0 {
-		return nil, nil
+		return nil, diags
 	}
 
-	return payload, nil
+	return payload, diags
 }
 
-func (c *RemoteClient) Put(data []byte) error {
+func (c *RemoteClient) Put(data []byte) tfdiags.Diagnostics {
+	var diags tfdiags.Diagnostics
+
 	getOptions := blobs.GetPropertiesInput{}
 	setOptions := blobs.SetPropertiesInput{}
 	putOptions := blobs.PutBlockBlobInput{}
@@ -73,23 +92,23 @@ func (c *RemoteClient) Put(data []byte) error {
 		putOptions.LeaseID = &c.leaseID
 	}
 
-	ctx := context.TODO()
+	ctx := newCtx()
 
 	if c.snapshot {
 		snapshotInput := blobs.SnapshotInput{LeaseID: options.LeaseID}
 
 		log.Printf("[DEBUG] Snapshotting existing Blob %q (Container %q / Account %q)", c.keyName, c.containerName, c.accountName)
-		if _, err := c.giovanniBlobClient.Snapshot(ctx, c.accountName, c.containerName, c.keyName, snapshotInput); err != nil {
-			return fmt.Errorf("error snapshotting Blob %q (Container %q / Account %q): %+v", c.keyName, c.containerName, c.accountName, err)
+		if _, err := c.giovanniBlobClient.Snapshot(ctx, c.containerName, c.keyName, snapshotInput); err != nil {
+			return diags.Append(fmt.Errorf("error snapshotting Blob %q (Container %q / Account %q): %+v", c.keyName, c.containerName, c.accountName, err))
 		}
 
 		log.Print("[DEBUG] Created blob snapshot")
 	}
 
-	blob, err := c.giovanniBlobClient.GetProperties(ctx, c.accountName, c.containerName, c.keyName, getOptions)
+	blob, err := c.giovanniBlobClient.GetProperties(ctx, c.containerName, c.keyName, getOptions)
 	if err != nil {
-		if blob.StatusCode != 404 {
-			return err
+		if !response.WasNotFound(blob.HttpResponse) {
+			return diags.Append(err)
 		}
 	}
 
@@ -97,26 +116,28 @@ func (c *RemoteClient) Put(data []byte) error {
 	putOptions.Content = &data
 	putOptions.ContentType = &contentType
 	putOptions.MetaData = blob.MetaData
-	_, err = c.giovanniBlobClient.PutBlockBlob(ctx, c.accountName, c.containerName, c.keyName, putOptions)
+	_, err = c.giovanniBlobClient.PutBlockBlob(ctx, c.containerName, c.keyName, putOptions)
 
-	return err
+	return diags.Append(err)
 }
 
-func (c *RemoteClient) Delete() error {
+func (c *RemoteClient) Delete() tfdiags.Diagnostics {
+	var diags tfdiags.Diagnostics
+
 	options := blobs.DeleteInput{}
 
 	if c.leaseID != "" {
 		options.LeaseID = &c.leaseID
 	}
 
-	ctx := context.TODO()
-	resp, err := c.giovanniBlobClient.Delete(ctx, c.accountName, c.containerName, c.keyName, options)
+	ctx := newCtx()
+	resp, err := c.giovanniBlobClient.Delete(ctx, c.containerName, c.keyName, options)
 	if err != nil {
-		if !resp.IsHTTPStatus(http.StatusNotFound) {
-			return err
+		if !response.WasNotFound(resp.HttpResponse) {
+			return diags.Append(err)
 		}
 	}
-	return nil
+	return diags
 }
 
 func (c *RemoteClient) Lock(info *statemgr.LockInfo) (string, error) {
@@ -135,7 +156,7 @@ func (c *RemoteClient) Lock(info *statemgr.LockInfo) (string, error) {
 	getLockInfoErr := func(err error) error {
 		lockInfo, infoErr := c.getLockInfo()
 		if infoErr != nil {
-			err = multierror.Append(err, infoErr)
+			err = errors.Join(err, infoErr)
 		}
 
 		return &statemgr.LockError{
@@ -148,13 +169,13 @@ func (c *RemoteClient) Lock(info *statemgr.LockInfo) (string, error) {
 		ProposedLeaseID: &info.ID,
 		LeaseDuration:   -1,
 	}
-	ctx := context.TODO()
+	ctx := newCtx()
 
 	// obtain properties to see if the blob lease is already in use. If the blob doesn't exist, create it
-	properties, err := c.giovanniBlobClient.GetProperties(ctx, c.accountName, c.containerName, c.keyName, blobs.GetPropertiesInput{})
+	properties, err := c.giovanniBlobClient.GetProperties(ctx, c.containerName, c.keyName, blobs.GetPropertiesInput{})
 	if err != nil {
 		// error if we had issues getting the blob
-		if !properties.Response.IsHTTPStatus(http.StatusNotFound) {
+		if !response.WasNotFound(properties.HttpResponse) {
 			return "", getLockInfoErr(err)
 		}
 		// if we don't find the blob, we need to build it
@@ -164,7 +185,7 @@ func (c *RemoteClient) Lock(info *statemgr.LockInfo) (string, error) {
 			ContentType: &contentType,
 		}
 
-		_, err = c.giovanniBlobClient.PutBlockBlob(ctx, c.accountName, c.containerName, c.keyName, putGOptions)
+		_, err = c.giovanniBlobClient.PutBlockBlob(ctx, c.containerName, c.keyName, putGOptions)
 		if err != nil {
 			return "", getLockInfoErr(err)
 		}
@@ -175,7 +196,7 @@ func (c *RemoteClient) Lock(info *statemgr.LockInfo) (string, error) {
 		return "", getLockInfoErr(fmt.Errorf("state blob is already locked"))
 	}
 
-	leaseID, err := c.giovanniBlobClient.AcquireLease(ctx, c.accountName, c.containerName, c.keyName, leaseOptions)
+	leaseID, err := c.giovanniBlobClient.AcquireLease(ctx, c.containerName, c.keyName, leaseOptions)
 	if err != nil {
 		return "", getLockInfoErr(err)
 	}
@@ -196,8 +217,8 @@ func (c *RemoteClient) getLockInfo() (*statemgr.LockInfo, error) {
 		options.LeaseID = &c.leaseID
 	}
 
-	ctx := context.TODO()
-	blob, err := c.giovanniBlobClient.GetProperties(ctx, c.accountName, c.containerName, c.keyName, options)
+	ctx := newCtx()
+	blob, err := c.giovanniBlobClient.GetProperties(ctx, c.containerName, c.keyName, options)
 	if err != nil {
 		return nil, err
 	}
@@ -223,8 +244,8 @@ func (c *RemoteClient) getLockInfo() (*statemgr.LockInfo, error) {
 
 // writes info to blob meta data, deletes metadata entry if info is nil
 func (c *RemoteClient) writeLockInfo(info *statemgr.LockInfo) error {
-	ctx := context.TODO()
-	blob, err := c.giovanniBlobClient.GetProperties(ctx, c.accountName, c.containerName, c.keyName, blobs.GetPropertiesInput{LeaseID: &c.leaseID})
+	ctx := newCtx()
+	blob, err := c.giovanniBlobClient.GetProperties(ctx, c.containerName, c.keyName, blobs.GetPropertiesInput{LeaseID: &c.leaseID})
 	if err != nil {
 		return err
 	}
@@ -244,7 +265,7 @@ func (c *RemoteClient) writeLockInfo(info *statemgr.LockInfo) error {
 		MetaData: blob.MetaData,
 	}
 
-	_, err = c.giovanniBlobClient.SetMetaData(ctx, c.accountName, c.containerName, c.keyName, opts)
+	_, err = c.giovanniBlobClient.SetMetaData(ctx, c.containerName, c.keyName, opts)
 	return err
 }
 
@@ -269,8 +290,8 @@ func (c *RemoteClient) Unlock(id string) error {
 		return lockErr
 	}
 
-	ctx := context.TODO()
-	_, err = c.giovanniBlobClient.ReleaseLease(ctx, c.accountName, c.containerName, c.keyName, id)
+	ctx := newCtx()
+	_, err = c.giovanniBlobClient.ReleaseLease(ctx, c.containerName, c.keyName, blobs.ReleaseLeaseInput{LeaseID: id})
 	if err != nil {
 		lockErr.Err = err
 		return lockErr
