@@ -1,3 +1,6 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package oss
 
 import (
@@ -15,6 +18,7 @@ import (
 	"github.com/hashicorp/terraform/internal/states"
 	"github.com/hashicorp/terraform/internal/states/remote"
 	"github.com/hashicorp/terraform/internal/states/statemgr"
+	"github.com/hashicorp/terraform/internal/tfdiags"
 )
 
 const (
@@ -49,17 +53,19 @@ func (b *Backend) remoteClient(name string) (*RemoteClient, error) {
 	return client, nil
 }
 
-func (b *Backend) Workspaces() ([]string, error) {
+func (b *Backend) Workspaces() ([]string, tfdiags.Diagnostics) {
+	var diags tfdiags.Diagnostics
+
 	bucket, err := b.ossClient.Bucket(b.bucketName)
 	if err != nil {
-		return []string{""}, fmt.Errorf("error getting bucket: %#v", err)
+		return []string{""}, diags.Append(fmt.Errorf("error getting bucket: %#v", err))
 	}
 
 	var options []oss.Option
 	options = append(options, oss.Prefix(b.statePrefix+"/"), oss.MaxKeys(1000))
 	resp, err := bucket.ListObjects(options...)
 	if err != nil {
-		return nil, err
+		return nil, diags.Append(err)
 	}
 
 	result := []string{backend.DefaultStateName}
@@ -85,37 +91,45 @@ func (b *Backend) Workspaces() ([]string, error) {
 				options = append(options, oss.Marker(lastObj))
 			}
 			resp, err = bucket.ListObjects(options...)
+			if err != nil {
+				return nil, diags.Append(err)
+			}
 		} else {
 			break
 		}
 	}
 	sort.Strings(result[1:])
-	return result, nil
+	return result, diags
 }
 
-func (b *Backend) DeleteWorkspace(name string) error {
+func (b *Backend) DeleteWorkspace(name string, _ bool) tfdiags.Diagnostics {
+	var diags tfdiags.Diagnostics
+
 	if name == backend.DefaultStateName || name == "" {
-		return fmt.Errorf("can't delete default state")
+		return diags.Append(fmt.Errorf("can't delete default state"))
 	}
 
 	client, err := b.remoteClient(name)
 	if err != nil {
-		return err
+		return diags.Append(err)
 	}
-	return client.Delete()
+	return diags.Append(client.Delete())
 }
 
-func (b *Backend) StateMgr(name string) (statemgr.Full, error) {
+func (b *Backend) StateMgr(name string) (statemgr.Full, tfdiags.Diagnostics) {
+	var diags tfdiags.Diagnostics
+
 	client, err := b.remoteClient(name)
 	if err != nil {
-		return nil, err
+		return nil, diags.Append(err)
 	}
 	stateMgr := &remote.State{Client: client}
 
 	// Check to see if this state already exists.
-	existing, err := b.Workspaces()
-	if err != nil {
-		return nil, err
+	existing, wDiags := b.Workspaces()
+	diags = diags.Append(wDiags)
+	if wDiags.HasErrors() {
+		return nil, diags
 	}
 
 	log.Printf("[DEBUG] Current workspace name: %s. All workspaces:%#v", name, existing)
@@ -134,7 +148,7 @@ func (b *Backend) StateMgr(name string) (statemgr.Full, error) {
 		lockInfo.Operation = "init"
 		lockId, err := client.Lock(lockInfo)
 		if err != nil {
-			return nil, fmt.Errorf("failed to lock OSS state: %s", err)
+			return nil, diags.Append(fmt.Errorf("failed to lock OSS state: %s", err))
 		}
 
 		// Local helper function so we can call it multiple places
@@ -148,28 +162,28 @@ func (b *Backend) StateMgr(name string) (statemgr.Full, error) {
 		// Grab the value
 		if err := stateMgr.RefreshState(); err != nil {
 			err = lockUnlock(err)
-			return nil, err
+			return nil, diags.Append(err)
 		}
 
 		// If we have no state, we have to create an empty state
 		if v := stateMgr.State(); v == nil {
 			if err := stateMgr.WriteState(states.NewState()); err != nil {
 				err = lockUnlock(err)
-				return nil, err
+				return nil, diags.Append(err)
 			}
-			if err := stateMgr.PersistState(); err != nil {
+			if err := stateMgr.PersistState(nil); err != nil {
 				err = lockUnlock(err)
-				return nil, err
+				return nil, diags.Append(err)
 			}
 		}
 
 		// Unlock, the state should now be initialized
 		if err := lockUnlock(nil); err != nil {
-			return nil, err
+			return nil, diags.Append(err)
 		}
 
 	}
-	return stateMgr, nil
+	return stateMgr, diags
 }
 
 func (b *Backend) stateFile(name string) string {
