@@ -7,14 +7,15 @@ import (
 	"bytes"
 	"encoding/json"
 	"fmt"
+	"slices"
 	"sort"
 	"strings"
 
-	"slices"
-
 	"github.com/hashicorp/terraform/internal/command/format"
 	"github.com/hashicorp/terraform/internal/command/jsonformat/computed"
 	"github.com/hashicorp/terraform/internal/command/jsonformat/computed/renderers"
+	"github.com/hashicorp/terraform/internal/command/jsonformat/differ"
+	"github.com/hashicorp/terraform/internal/command/jsonformat/structured"
 	"github.com/hashicorp/terraform/internal/command/jsonplan"
 	"github.com/hashicorp/terraform/internal/command/jsonprovider"
 	"github.com/hashicorp/terraform/internal/command/jsonstate"
@@ -33,6 +34,7 @@ type Plan struct {
 	ResourceDrift      []jsonplan.ResourceChange         `json:"resource_drift,omitempty"`
 	RelevantAttributes []jsonplan.ResourceAttr           `json:"relevant_attributes,omitempty"`
 	DeferredChanges    []jsonplan.DeferredResourceChange `json:"deferred_changes,omitempty"`
+	ActionInvocations  []jsonplan.ActionInvocation       `json:"action_invocations,omitempty"`
 
 	ProviderFormatVersion string                            `json:"provider_format_version"`
 	ProviderSchemas       map[string]*jsonprovider.Provider `json:"provider_schemas,omitempty"`
@@ -49,6 +51,10 @@ func (plan Plan) getSchema(change jsonplan.ResourceChange) *jsonprovider.Schema
 	}
 }
 
+func (plan Plan) getActionSchema(ai jsonplan.ActionInvocation) *jsonprovider.ActionSchema {
+	return plan.ProviderSchemas[ai.ProviderName].ActionSchemas[ai.Type]
+}
+
 func (plan Plan) renderHuman(renderer Renderer, mode plans.Mode, opts ...plans.Quality) {
 	checkOpts := func(target plans.Quality) bool {
 		return slices.Contains(opts, target)
@@ -85,11 +91,13 @@ func (plan Plan) renderHuman(renderer Renderer, mode plans.Mode, opts ...plans.Q
 		}
 	}
 
-	// Precompute the outputs early, so we can make a decision about whether we
+	// Precompute the outputs and actions early, so we can make a decision about whether we
 	// display the "there are no changes messages".
 	outputs := renderHumanDiffOutputs(renderer, diffs.outputs)
+	actions := renderHumanActionInvocations(renderer, diffs.actions)
+	actionCount := len(plan.ActionInvocations) // diffs.actions is just the CLI-invoked actions
 
-	if len(changes) == 0 && len(outputs) == 0 {
+	if len(changes) == 0 && len(outputs) == 0 && actionCount == 0 {
 		// If we didn't find any changes to report at all then this is a
 		// "No changes" plan. How we'll present this depends on whether
 		// the plan is "applyable" and, if so, whether it had refresh changes
@@ -217,7 +225,7 @@ func (plan Plan) renderHuman(renderer Renderer, mode plans.Mode, opts ...plans.Q
 		}
 	}
 
-	if len(changes) > 0 {
+	if len(changes) > 0 || actionCount > 0 {
 		if checkOpts(plans.Errored) {
 			renderer.Streams.Printf("\nTerraform planned the following actions, but then encountered a problem:\n")
 		} else {
@@ -232,20 +240,28 @@ func (plan Plan) renderHuman(renderer Renderer, mode plans.Mode, opts ...plans.Q
 			}
 		}
 
+		var buf strings.Builder
+		buf.WriteString(renderer.Colorize.Color("\n[bold]Plan:[reset] "))
 		if importingCount > 0 {
-			renderer.Streams.Printf(
-				renderer.Colorize.Color("\n[bold]Plan:[reset] %d to import, %d to add, %d to change, %d to destroy.\n"),
-				importingCount,
-				counts[plans.Create]+counts[plans.DeleteThenCreate]+counts[plans.CreateThenDelete],
-				counts[plans.Update],
-				counts[plans.Delete]+counts[plans.DeleteThenCreate]+counts[plans.CreateThenDelete])
-		} else {
-			renderer.Streams.Printf(
-				renderer.Colorize.Color("\n[bold]Plan:[reset] %d to add, %d to change, %d to destroy.\n"),
-				counts[plans.Create]+counts[plans.DeleteThenCreate]+counts[plans.CreateThenDelete],
-				counts[plans.Update],
-				counts[plans.Delete]+counts[plans.DeleteThenCreate]+counts[plans.CreateThenDelete])
+			buf.WriteString(fmt.Sprintf("%d to import, ", importingCount))
 		}
+		buf.WriteString(fmt.Sprintf("%d to add, %d to change, %d to destroy.",
+			counts[plans.Create]+counts[plans.DeleteThenCreate]+counts[plans.CreateThenDelete],
+			counts[plans.Update],
+			counts[plans.Delete]+counts[plans.DeleteThenCreate]+counts[plans.CreateThenDelete]),
+		)
+
+		if actionCount > 0 {
+			buf.WriteString(fmt.Sprintf(" Actions: %d to invoke.", actionCount))
+		}
+		buf.WriteString("\n")
+
+		renderer.Streams.Print(buf.String())
+	}
+
+	if len(actions) > 0 {
+		renderer.Streams.Print(renderer.Colorize.Color("\nTerraform will invoke the following action(s):\n\n"))
+		renderer.Streams.Printf("%s\n", actions)
 	}
 
 	if len(outputs) > 0 {
@@ -387,9 +403,65 @@ func renderHumanDiff(renderer Renderer, diff diff, cause string) (string, bool)
 	opts.ShowUnchangedChildren = diff.Importing()
 
 	buf.WriteString(fmt.Sprintf("%s %s %s", renderer.Colorize.Color(format.DiffActionSymbol(action)), resourceChangeHeader(diff.change), diff.diff.RenderHuman(0, opts)))
+
+	if len(diff.beforeActionsTriggered) > 0 {
+		buf.WriteString(renderer.Colorize.Color("\n\n    [bold]# Actions to be invoked before this change in order:[reset]\n"))
+		for _, ai := range diff.beforeActionsTriggered {
+			buf.WriteString(renderActionInvocation(renderer, ai))
+		}
+	}
+
+	if len(diff.afterActionsTriggered) > 0 {
+		buf.WriteString(renderer.Colorize.Color("\n\n    [bold]# Actions to be invoked after this change in order:[reset]\n"))
+		for _, ai := range diff.afterActionsTriggered {
+			buf.WriteString(renderActionInvocation(renderer, ai))
+		}
+	}
+
 	return buf.String(), true
 }
 
+func renderActionInvocation(renderer Renderer, ai actionInvocation) string {
+	var buf bytes.Buffer
+	buf.WriteString(fmt.Sprintf("    action %q %q {\n", ai.invocation.Type, ai.invocation.Name))
+	if len(ai.invocation.ConfigValues) > 0 {
+		buf.WriteString("        config ")
+
+		opts := computed.NewRenderHumanOpts(renderer.Colorize)
+		opts.ShowUnchangedChildren = true
+		opts.HideDiffActionSymbols = true
+		change := structured.FromJsonActionInvocation(ai.invocation)
+
+		buf.WriteString(indentExceptFirstLine(differ.ComputeDiffForBlock(change, ai.schema.ConfigSchema).RenderHuman(0, opts), 8))
+		buf.WriteString("\n")
+	}
+	buf.WriteString("    }\n")
+	return buf.String()
+}
+
+func indentExceptFirstLine(s string, indentation int) string {
+	lines := strings.Split(s, "\n")
+	if len(lines) == 0 {
+		return s
+	}
+	var buf bytes.Buffer
+	for i, line := range lines {
+		if i == 0 {
+			buf.WriteString(line)
+		} else {
+			// Indent all lines except the first one
+			buf.WriteString(strings.Repeat(" ", indentation))
+			buf.WriteString(line)
+		}
+
+		// Add a newline after each line except for the last one
+		if i < len(lines)-1 {
+			buf.WriteString("\n")
+		}
+	}
+	return buf.String()
+}
+
 func renderHumanDeferredDiff(renderer Renderer, deferred deferredDiff) (string, bool) {
 
 	// Internally, our computed diffs can't tell the difference between a
@@ -432,6 +504,17 @@ func renderHumanDeferredDiff(renderer Renderer, deferred deferredDiff) (string,
 	return buf.String(), true
 }
 
+// All actions that run based on the resource lifecycle should be rendered as part of the resource
+// changes, therefore this function only renders actions that are invoked by the CLI
+func renderHumanActionInvocations(renderer Renderer, actionInvocations []actionInvocation) string {
+	var invocations []string
+	for _, invocation := range actionInvocations {
+		header := fmt.Sprintf(renderer.Colorize.Color("  [bold]# %s[reset] will be invoked"), invocation.invocation.Address)
+		invocations = append(invocations, fmt.Sprintf("%s\n%s", header, renderActionInvocation(renderer, invocation)))
+	}
+	return strings.Join(invocations, "\n")
+}
+
 func resourceChangeComment(resource jsonplan.ResourceChange, action plans.Action, changeCause string) string {
 	var buf bytes.Buffer
 
