@@ -30,6 +30,7 @@ type Provider struct {
 	ListResourceSchemas      map[string]*Schema                         `json:"list_resource_schemas,omitempty"`
 	Functions                map[string]*jsonfunction.FunctionSignature `json:"functions,omitempty"`
 	ResourceIdentitySchemas  map[string]*IdentitySchema                 `json:"resource_identity_schemas,omitempty"`
+	ActionSchemas            map[string]*ActionSchema                   `json:"action_schemas,omitempty"`
 }
 
 func newProviders() *Providers {
@@ -44,22 +45,22 @@ func newProviders() *Providers {
 // schema into the public structured JSON versions.
 //
 // This is a format that can be read by the structured plan renderer.
-func MarshalForRenderer(s *terraform.Schemas, includeExperimentalSchemas bool) map[string]*Provider {
+func MarshalForRenderer(s *terraform.Schemas) map[string]*Provider {
 	schemas := make(map[string]*Provider, len(s.Providers))
 	for k, v := range s.Providers {
-		schemas[k.String()] = marshalProvider(v, includeExperimentalSchemas)
+		schemas[k.String()] = marshalProvider(v)
 	}
 	return schemas
 }
 
-func Marshal(s *terraform.Schemas, includeExperimentalSchemas bool) ([]byte, error) {
+func Marshal(s *terraform.Schemas) ([]byte, error) {
 	providers := newProviders()
-	providers.Schemas = MarshalForRenderer(s, includeExperimentalSchemas)
+	providers.Schemas = MarshalForRenderer(s)
 	ret, err := json.Marshal(providers)
 	return ret, err
 }
 
-func marshalProvider(tps providers.ProviderSchema, includeExperimentalSchemas bool) *Provider {
+func marshalProvider(tps providers.ProviderSchema) *Provider {
 	p := &Provider{
 		Provider:                 marshalSchema(tps.Provider),
 		ResourceSchemas:          marshalSchemas(tps.ResourceTypes),
@@ -67,11 +68,21 @@ func marshalProvider(tps providers.ProviderSchema, includeExperimentalSchemas bo
 		EphemeralResourceSchemas: marshalSchemas(tps.EphemeralResourceTypes),
 		Functions:                jsonfunction.MarshalProviderFunctions(tps.Functions),
 		ResourceIdentitySchemas:  marshalIdentitySchemas(tps.ResourceTypes),
+		ActionSchemas:            marshalActionSchemas(tps.Actions),
 	}
 
-	if includeExperimentalSchemas {
-		p.ListResourceSchemas = marshalSchemas(tps.ListResourceTypes)
+	// List resource schemas are nested under a "config" block, so we need to
+	// extract that block to get the actual provider schema for the list resource.
+	// When getting the provider schemas, Terraform adds this extra level to
+	// better match the actual configuration structure.
+	listSchemas := make(map[string]providers.Schema, len(tps.ListResourceTypes))
+	for k, v := range tps.ListResourceTypes {
+		listSchemas[k] = providers.Schema{
+			Body:    &v.Body.BlockTypes["config"].Block,
+			Version: v.Version,
+		}
 	}
+	p.ListResourceSchemas = marshalSchemas(listSchemas)
 
 	return p
 }
