@@ -83,9 +83,9 @@ func (c *OutputCommand) Outputs(statePath string) (map[string]*states.OutputValu
 	}
 
 	// Get the state
-	stateStore, err := b.StateMgr(env)
-	if err != nil {
-		diags = diags.Append(fmt.Errorf("Failed to load state: %s", err))
+	stateStore, sDiags := b.StateMgr(env)
+	if sDiags.HasErrors() {
+		diags = diags.Append(fmt.Errorf("Failed to load state: %s", sDiags.Err()))
 		return nil, diags
 	}
 
@@ -109,8 +109,10 @@ Usage: terraform [global options] output [options] [NAME]
 Options:
 
   -state=path      Path to the state file to read. Defaults to
-                   "terraform.tfstate". Ignored when remote 
-                   state is used.
+                   "terraform.tfstate".
+                   Legacy option for the local backend only. See
+                   the local backend's documentation for more
+                   information.
 
   -no-color        If specified, output won't contain any color.
 
