@@ -43,7 +43,7 @@ func (p *Parser) LoadTestFile(path string) (*TestFile, hcl.Diagnostics) {
 		return nil, diags
 	}
 
-	test, testDiags := loadTestFile(body)
+	test, testDiags := loadTestFile(body, p.allowExperiments)
 	diags = append(diags, testDiags...)
 	return test, diags
 }
@@ -104,14 +104,6 @@ func parseConfigFile(body hcl.Body, diags hcl.Diagnostics, override, allowExperi
 		switch block.Type {
 
 		case "terraform":
-			// TODO: Update once pluggable state store is out of experimental phase
-			if allowExperiments {
-				terraformBlockSchema.Blocks = append(terraformBlockSchema.Blocks,
-					hcl.BlockHeaderSchema{
-						Type:       "state_store",
-						LabelNames: []string{"type"},
-					})
-			}
 			content, contentDiags := block.Body.Content(terraformBlockSchema)
 			diags = append(diags, contentDiags...)
 
@@ -130,10 +122,20 @@ func parseConfigFile(body hcl.Body, diags hcl.Diagnostics, override, allowExperi
 					}
 
 				case "state_store":
-					stateStoreCfg, cfgDiags := decodeStateStoreBlock(innerBlock)
-					diags = append(diags, cfgDiags...)
-					if stateStoreCfg != nil {
-						file.StateStores = append(file.StateStores, stateStoreCfg)
+					if allowExperiments {
+						stateStoreCfg, cfgDiags := decodeStateStoreBlock(innerBlock)
+						diags = append(diags, cfgDiags...)
+						if stateStoreCfg != nil {
+							file.StateStores = append(file.StateStores, stateStoreCfg)
+						}
+					} else {
+						// Prevent parsing of state_store blocks in all commands unless experiments enabled.
+						diags = diags.Append(&hcl.Diagnostic{
+							Severity: hcl.DiagError,
+							Summary:  "Unsupported block type",
+							Detail:   "Blocks of type \"state_store\" are not expected here.",
+							Subject:  &innerBlock.TypeRange,
+						})
 					}
 				case "cloud":
 					cloudCfg, cfgDiags := decodeCloudBlock(innerBlock)
@@ -256,12 +258,10 @@ func parseConfigFile(body hcl.Body, diags hcl.Diagnostics, override, allowExperi
 			}
 
 		case "action":
-			if allowExperiments {
-				cfg, cfgDiags := decodeActionBlock(block)
-				diags = append(diags, cfgDiags...)
-				if cfg != nil {
-					file.Actions = append(file.Actions, cfg)
-				}
+			cfg, cfgDiags := decodeActionBlock(block)
+			diags = append(diags, cfgDiags...)
+			if cfg != nil {
+				file.Actions = append(file.Actions, cfg)
 			}
 
 		default:
@@ -394,6 +394,10 @@ var terraformBlockSchema = &hcl.BodySchema{
 		{
 			Type: "required_providers",
 		},
+		{
+			Type:       "state_store",
+			LabelNames: []string{"type"},
+		},
 		{
 			Type:       "provider_meta",
 			LabelNames: []string{"provider"},
