@@ -298,14 +298,11 @@ func decodeResourceBlock(block *hcl.Block, override bool, allowExperiments bool)
 						r.Postconditions = append(r.Postconditions, cr)
 					}
 
-				// decoded, but not yet used!
 				case "action_trigger":
-					if allowExperiments {
-						at, atDiags := decodeActionTriggerBlock(block)
-						diags = append(diags, atDiags...)
-						if at != nil {
-							r.Managed.ActionTriggers = append(r.Managed.ActionTriggers, at)
-						}
+					at, atDiags := decodeActionTriggerBlock(block)
+					diags = append(diags, atDiags...)
+					if at != nil {
+						r.Managed.ActionTriggers = append(r.Managed.ActionTriggers, at)
 					}
 
 				default:
