@@ -40,6 +40,12 @@ const (
 	RefreshOnlyTestMode TestMode = 'R'
 )
 
+const (
+	// TestMainStateIdentifier is the default state identifier for the main
+	// state of the test file.
+	TestMainStateIdentifier = ""
+)
+
 // TestFile represents a single test file within a `terraform test` execution.
 //
 // A test file is made up of a sequential list of run blocks, each designating
@@ -66,6 +72,11 @@ type TestFile struct {
 	// test.
 	Providers map[string]*Provider
 
+	// BackendConfigs is a map of state keys to structs that contain backend
+	// configuration. This should be used to set the state for a given state key
+	// at the start of a test command.
+	BackendConfigs map[string]RunBlockBackend
+
 	// Overrides contains any specific overrides that should be applied for this
 	// test outside any mock providers.
 	Overrides addrs.Map[addrs.Targetable, *Override]
@@ -84,6 +95,9 @@ type TestFileConfig struct {
 	// Parallel: Indicates if test runs should be executed in parallel.
 	Parallel bool
 
+	// SkipCleanup: Indicates if the test runs should skip the cleanup phase.
+	SkipCleanup bool
+
 	DeclRange hcl.Range
 }
 
@@ -164,6 +178,12 @@ type TestRun struct {
 	// will be executed in parallel with other test runs.
 	Parallel bool
 
+	Backend *Backend
+
+	// SkipCleanup: Indicates if the test run should skip the cleanup phase.
+	SkipCleanup      bool
+	SkipCleanupRange *hcl.Range
+
 	NameDeclRange      hcl.Range
 	VariablesDeclRange hcl.Range
 	DeclRange          hcl.Range
@@ -332,11 +352,24 @@ type TestRunOptions struct {
 	DeclRange hcl.Range
 }
 
-func loadTestFile(body hcl.Body) (*TestFile, hcl.Diagnostics) {
+// RunBlockBackend records a backend block and which run block it was parsed
+// from.
+type RunBlockBackend struct {
+	Backend *Backend
+
+	// Run is the TestRun containing the backend block for this Backend.
+	// This is used in diagnostics to help avoid duplicate backends for a given
+	// internal state file or duplicated use of the same backend for multiple
+	// internal states.
+	Run *TestRun
+}
+
+func loadTestFile(body hcl.Body, experimentsAllowed bool) (*TestFile, hcl.Diagnostics) {
 	var diags hcl.Diagnostics
 	tf := &TestFile{
 		VariableDefinitions: make(map[string]*Variable),
 		Providers:           make(map[string]*Provider),
+		BackendConfigs:      make(map[string]RunBlockBackend),
 		Overrides:           addrs.MakeMap[addrs.Targetable, *Override](),
 	}
 
@@ -348,7 +381,7 @@ func loadTestFile(body hcl.Body) (*TestFile, hcl.Diagnostics) {
 	diags = append(diags, contentDiags...)
 
 	var cDiags hcl.Diagnostics
-	tf.Config, cDiags = decodeFileConfigBlock(configContent)
+	tf.Config, cDiags = decodeFileConfigBlock(configContent, experimentsAllowed)
 	diags = append(diags, cDiags...)
 	if diags.HasErrors() {
 		return nil, diags
@@ -358,11 +391,14 @@ func loadTestFile(body hcl.Body) (*TestFile, hcl.Diagnostics) {
 	diags = append(diags, contentDiags...)
 
 	runBlockNames := make(map[string]hcl.Range)
+	skipCleanups := make(map[string]string)
 
 	for _, block := range content.Blocks {
 		switch block.Type {
 		case "run":
-			run, runDiags := decodeTestRunBlock(block, tf)
+			nextRunIndex := len(tf.Runs)
+
+			run, runDiags := decodeTestRunBlock(block, tf, experimentsAllowed)
 			diags = append(diags, runDiags...)
 			if !runDiags.HasErrors() {
 				tf.Runs = append(tf.Runs, run)
@@ -373,11 +409,71 @@ func loadTestFile(body hcl.Body) (*TestFile, hcl.Diagnostics) {
 					Severity: hcl.DiagError,
 					Summary:  "Duplicate \"run\" block names",
 					Detail:   fmt.Sprintf("This test file already has a run named %s block defined at %s.", run.Name, rng),
-					Subject:  block.DefRange.Ptr(),
+					Subject:  run.NameDeclRange.Ptr(),
 				})
-				continue
+			} else {
+				runBlockNames[run.Name] = run.DeclRange
+			}
+
+			if run.SkipCleanup && run.SkipCleanupRange != nil {
+				if backend, found := tf.BackendConfigs[run.StateKey]; found {
+					diags = append(diags, &hcl.Diagnostic{
+						Severity: hcl.DiagWarning,
+						Summary:  "Duplicate \"skip_cleanup\" block",
+						Detail:   fmt.Sprintf("The run %q has a skip_cleanup attribute set, but shares state with an earlier run %q that has a backend defined. The later run takes precedence, but the backend will still be used to manage this state.", run.Name, backend.Run.Name),
+						Subject:  run.SkipCleanupRange,
+					})
+				} else {
+					if _, found := skipCleanups[run.StateKey]; found {
+						diags = append(diags, &hcl.Diagnostic{
+							Severity: hcl.DiagWarning,
+							Summary:  "Duplicate \"skip_cleanup\" block",
+							Detail:   fmt.Sprintf("The run %q has a skip_cleanup attribute set, but shares state with an earlier run %q that also has skip_cleanup set. The later run takes precedence, and this attribute is ignored for the earlier run.", run.Name, skipCleanups[run.StateKey]),
+							Subject:  run.SkipCleanupRange,
+						})
+					}
+					skipCleanups[run.StateKey] = run.Name
+				}
+			}
+
+			if run.Backend != nil {
+				if existing, exists := tf.BackendConfigs[run.StateKey]; exists {
+					// then we definitely have two run blocks with the same
+					// state key trying to load backends
+					diags = append(diags, &hcl.Diagnostic{
+						Severity: hcl.DiagError,
+						Summary:  "Duplicate backend blocks",
+						Detail:   fmt.Sprintf("The run %q already uses an internal state file that's loaded by a backend in the run %q. Please ensure that a backend block is only in the first apply run block for a given internal state file.", run.Name, existing.Run.Name),
+						Subject:  run.Backend.DeclRange.Ptr(),
+					})
+					continue
+				} else {
+					// Record the backend block in the test file, under the related state key
+					tf.BackendConfigs[run.StateKey] = RunBlockBackend{
+						Backend: run.Backend,
+						Run:     run,
+					}
+				}
+
+				for ix := range nextRunIndex {
+					previousRun := tf.Runs[ix]
+
+					if previousRun.StateKey != run.StateKey {
+						continue
+					}
+
+					if previousRun.Command == ApplyTestCommand {
+						diags = append(diags, &hcl.Diagnostic{
+							Severity: hcl.DiagError,
+							Summary:  "Invalid backend block",
+							Detail:   fmt.Sprintf("The run %q cannot load in state using a backend block, because internal state has already been created by an apply command in run %q. Backend blocks can only be present in the first apply command for a given internal state.", run.Name, previousRun.Name),
+							Subject:  run.Backend.DeclRange.Ptr(),
+						})
+						break
+					}
+				}
+
 			}
-			runBlockNames[run.Name] = run.DeclRange
 
 		case "variable":
 			variable, variableDiags := decodeVariableBlock(block, false)
@@ -521,7 +617,7 @@ func loadTestFile(body hcl.Body) (*TestFile, hcl.Diagnostics) {
 	return tf, diags
 }
 
-func decodeFileConfigBlock(fileContent *hcl.BodyContent) (*TestFileConfig, hcl.Diagnostics) {
+func decodeFileConfigBlock(fileContent *hcl.BodyContent, experimentsAllowed bool) (*TestFileConfig, hcl.Diagnostics) {
 	var diags hcl.Diagnostics
 
 	// The "test" block is optional, so we just return a nil config if it doesn't exist.
@@ -555,10 +651,24 @@ func decodeFileConfigBlock(fileContent *hcl.BodyContent) (*TestFileConfig, hcl.D
 		diags = append(diags, rawDiags...)
 	}
 
+	if attr, exists := content.Attributes["skip_cleanup"]; exists {
+		if !experimentsAllowed {
+			diags = append(diags, &hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Invalid attribute",
+				Detail:   "The skip_cleanup attribute is only available in experimental builds of Terraform.",
+				Subject:  attr.NameRange.Ptr(),
+			})
+		}
+
+		rawDiags := gohcl.DecodeExpression(attr.Expr, nil, &ret.SkipCleanup)
+		diags = append(diags, rawDiags...)
+	}
+
 	return ret, diags
 }
 
-func decodeTestRunBlock(block *hcl.Block, file *TestFile) (*TestRun, hcl.Diagnostics) {
+func decodeTestRunBlock(block *hcl.Block, file *TestFile, experimentsAllowed bool) (*TestRun, hcl.Diagnostics) {
 	var diags hcl.Diagnostics
 
 	content, contentDiags := block.Body.Content(testRunBlockSchema)
@@ -571,6 +681,7 @@ func decodeTestRunBlock(block *hcl.Block, file *TestFile) (*TestRun, hcl.Diagnos
 		NameDeclRange: block.LabelRanges[0],
 		DeclRange:     block.DefRange,
 		Parallel:      file.Config != nil && file.Config.Parallel,
+		SkipCleanup:   file.Config != nil && file.Config.SkipCleanup,
 	}
 
 	if !hclsyntax.ValidIdentifier(r.Name) {
@@ -582,6 +693,7 @@ func decodeTestRunBlock(block *hcl.Block, file *TestFile) (*TestRun, hcl.Diagnos
 		})
 	}
 
+	var backendRange *hcl.Range // Stored for validation once all blocks/attrs processed
 	for _, block := range content.Blocks {
 		switch block.Type {
 		case "assert":
@@ -691,6 +803,45 @@ func decodeTestRunBlock(block *hcl.Block, file *TestFile) (*TestRun, hcl.Diagnos
 				}
 				r.Overrides.Put(subject, override)
 			}
+		case "backend":
+			if !experimentsAllowed {
+				diags = append(diags, &hcl.Diagnostic{
+					Severity: hcl.DiagError,
+					Summary:  "Invalid block",
+					Detail:   "The backend block is only available within run blocks in experimental builds of Terraform.",
+					Subject:  block.DefRange.Ptr(),
+				})
+			}
+
+			backend, backedDiags := decodeBackendBlock(block)
+			diags = append(diags, backedDiags...)
+
+			if backend.Type == "remote" {
+				// Enhanced backends are not in use
+				diags = append(diags, &hcl.Diagnostic{
+					Severity: hcl.DiagError,
+					Summary:  "Invalid backend block",
+					Detail:   fmt.Sprintf("The \"remote\" backend type cannot be used in the backend block in run %q at %s. Only state storage backends can be used in a test run.", r.Name, block.DefRange),
+					Subject:  block.DefRange.Ptr(),
+				})
+				continue
+			}
+
+			if r.Backend != nil {
+				// We've already encountered a backend for this run block
+				diags = append(diags, &hcl.Diagnostic{
+					Severity: hcl.DiagError,
+					Summary:  "Duplicate backend blocks",
+					Detail:   fmt.Sprintf("A backend block has already been defined inside the run %q at %s.", r.Name, backendRange),
+					Subject:  block.DefRange.Ptr(),
+				})
+				continue
+			}
+
+			r.Backend = backend
+			backendRange = &block.DefRange
+			// Using a backend implies skipping cleanup for that run
+			r.SkipCleanup = true
 		}
 	}
 
@@ -743,6 +894,10 @@ func decodeTestRunBlock(block *hcl.Block, file *TestFile) (*TestRun, hcl.Diagnos
 	if attr, exists := content.Attributes["state_key"]; exists {
 		rawDiags := gohcl.DecodeExpression(attr.Expr, nil, &r.StateKey)
 		diags = append(diags, rawDiags...)
+	} else if r.Module != nil {
+		r.StateKey = r.Module.Source.String()
+	} else {
+		r.StateKey = TestMainStateIdentifier // redundant, but let's be explicit
 	}
 
 	if attr, exists := content.Attributes["parallel"]; exists {
@@ -750,6 +905,42 @@ func decodeTestRunBlock(block *hcl.Block, file *TestFile) (*TestRun, hcl.Diagnos
 		diags = append(diags, rawDiags...)
 	}
 
+	if r.Command != ApplyTestCommand && r.Backend != nil {
+		// Backend blocks must be used in the first _apply_ run block for a given internal state file.
+		// So, they cannot be present in a plan run block
+		diags = append(diags, &hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Invalid backend block",
+			Detail:   "A backend block can only be used in the first apply run block for a given internal state file. It cannot be included in a block to run a plan command.",
+			Subject:  backendRange.Ptr(),
+		})
+	}
+
+	if attr, exists := content.Attributes["skip_cleanup"]; exists {
+		if !experimentsAllowed {
+			diags = append(diags, &hcl.Diagnostic{
+				Severity: hcl.DiagError,
+				Summary:  "Invalid attribute",
+				Detail:   "The skip_cleanup attribute is only available in experimental builds of Terraform.",
+				Subject:  attr.NameRange.Ptr(),
+			})
+		}
+
+		rawDiags := gohcl.DecodeExpression(attr.Expr, nil, &r.SkipCleanup)
+		diags = append(diags, rawDiags...)
+		r.SkipCleanupRange = attr.NameRange.Ptr()
+	}
+
+	if r.SkipCleanupRange != nil && !r.SkipCleanup && r.Backend != nil {
+		// Stop user attempting to clean up long-lived resources
+		diags = append(diags, &hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Cannot use `skip_cleanup=false` in a run block that contains a backend block",
+			Detail:   "Backend blocks are used in tests to allow reuse of long-lived resources. Due to this, cleanup behavior is implicitly skipped and backend blocks are incompatible with setting `skip_cleanup=false`",
+			Subject:  r.SkipCleanupRange,
+		})
+	}
+
 	return &r, diags
 }
 
@@ -953,6 +1144,7 @@ var testFileSchema = &hcl.BodySchema{
 var testFileConfigBlockSchema = &hcl.BodySchema{
 	Attributes: []hcl.AttributeSchema{
 		{Name: "parallel"},
+		{Name: "skip_cleanup"},
 	},
 }
 
@@ -963,6 +1155,7 @@ var testRunBlockSchema = &hcl.BodySchema{
 		{Name: "expect_failures"},
 		{Name: "state_key"},
 		{Name: "parallel"},
+		{Name: "skip_cleanup"},
 	},
 	Blocks: []hcl.BlockHeaderSchema{
 		{
@@ -986,6 +1179,10 @@ var testRunBlockSchema = &hcl.BodySchema{
 		{
 			Type: "override_module",
 		},
+		{
+			Type:       "backend",
+			LabelNames: []string{"name"},
+		},
 	},
 }
 
