@@ -25,13 +25,11 @@ const (
 	PreconditionsPostconditions    = Experiment("preconditions_postconditions")
 	EphemeralValues                = Experiment("ephemeral_values")
 	UnknownInstances               = Experiment("unknown_instances")
-	Actions                        = Experiment("actions")
 )
 
 func init() {
 	// Each experiment constant defined above must be registered here as either
 	// a current or a concluded experiment.
-	registerCurrentExperiment(Actions)
 	registerConcludedExperiment(UnknownInstances, "Unknown instances are being rolled into a larger feature for deferring unready resources and modules.")
 	registerConcludedExperiment(VariableValidation, "Custom variable validation can now be used by default, without enabling an experiment.")
 	registerConcludedExperiment(VariableValidationCrossRef, "Input variable validation rules may now refer to other objects in the same module without enabling any experiment.")
