@@ -7,10 +7,11 @@ import (
 	"fmt"
 	"testing"
 
+	"github.com/zclconf/go-cty/cty"
+
 	"github.com/hashicorp/terraform/internal/configs/configschema"
 	"github.com/hashicorp/terraform/internal/lang/marks"
 	"github.com/hashicorp/terraform/internal/providers"
-	"github.com/zclconf/go-cty/cty"
 )
 
 func TestChangeEncodeSensitive(t *testing.T) {
@@ -98,7 +99,7 @@ func TestChangeEncodeError(t *testing.T) {
 				Change: Change{
 					Before: cty.NullVal(cty.DynamicPseudoType),
 					// can't encode a marked value
-					After: cty.StringVal("test").Mark("shoult not be here"),
+					After: cty.StringVal("test").Mark("should not be here"),
 				},
 			},
 		},
