@@ -15,6 +15,7 @@ import (
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/checks"
 	"github.com/hashicorp/terraform/internal/collections"
+	"github.com/hashicorp/terraform/internal/configs"
 	"github.com/hashicorp/terraform/internal/configs/configschema"
 	"github.com/hashicorp/terraform/internal/lang/globalref"
 	"github.com/hashicorp/terraform/internal/plans"
@@ -176,6 +177,11 @@ func examplePlanForTest(t *testing.T) *plans.Plan {
 	applyTimeVariables := collections.NewSetCmp[string]()
 	applyTimeVariables.Add("bar")
 
+	provider := addrs.AbsProviderConfig{
+		Provider: addrs.NewDefaultProvider("test"),
+		Module:   addrs.RootModule,
+	}
+
 	return &plans.Plan{
 		Applyable: true,
 		Complete:  true,
@@ -224,10 +230,7 @@ func examplePlanForTest(t *testing.T) *plans.Plan {
 						Type: "test_thing",
 						Name: "woot",
 					}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-					ProviderAddr: addrs.AbsProviderConfig{
-						Provider: addrs.NewDefaultProvider("test"),
-						Module:   addrs.RootModule,
-					},
+					ProviderAddr: provider,
 					ChangeSrc: plans.ChangeSrc{
 						Action: plans.DeleteThenCreate,
 						Before: mustNewDynamicValue(cty.ObjectVal(map[string]cty.Value{
@@ -286,10 +289,7 @@ func examplePlanForTest(t *testing.T) *plans.Plan {
 						Type: "test_thing",
 						Name: "importing",
 					}.Instance(addrs.IntKey(1)).Absolute(addrs.RootModuleInstance),
-					ProviderAddr: addrs.AbsProviderConfig{
-						Provider: addrs.NewDefaultProvider("test"),
-						Module:   addrs.RootModule,
-					},
+					ProviderAddr: provider,
 					ChangeSrc: plans.ChangeSrc{
 						Action: plans.NoOp,
 						Before: mustNewDynamicValue(cty.ObjectVal(map[string]cty.Value{
@@ -303,6 +303,57 @@ func examplePlanForTest(t *testing.T) *plans.Plan {
 					},
 				},
 			},
+			ActionInvocations: []*plans.ActionInvocationInstanceSrc{
+				{
+					Addr:         addrs.Action{Type: "example", Name: "foo"}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
+					ProviderAddr: provider,
+					ActionTrigger: &plans.LifecycleActionTrigger{
+						ActionTriggerEvent:      configs.BeforeCreate,
+						ActionTriggerBlockIndex: 2,
+						ActionsListIndex:        0,
+						TriggeringResourceAddr: addrs.Resource{
+							Mode: addrs.ManagedResourceMode,
+							Type: "test_thing",
+							Name: "woot",
+						}.Instance(addrs.IntKey(0)).Absolute(addrs.RootModuleInstance),
+					},
+				},
+				{
+					Addr:         addrs.Action{Type: "example", Name: "bar"}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
+					ProviderAddr: provider,
+					ActionTrigger: &plans.LifecycleActionTrigger{
+						ActionTriggerEvent:      configs.BeforeCreate,
+						ActionTriggerBlockIndex: 2,
+						ActionsListIndex:        1,
+						TriggeringResourceAddr: addrs.Resource{
+							Mode: addrs.ManagedResourceMode,
+							Type: "test_thing",
+							Name: "woot",
+						}.Instance(addrs.IntKey(0)).Absolute(addrs.RootModuleInstance),
+					},
+					ConfigValue: mustNewDynamicValue(cty.ObjectVal(map[string]cty.Value{
+						"id": cty.StringVal("testing"),
+					}), objTy),
+				},
+				{
+					Addr:         addrs.Action{Type: "example", Name: "baz"}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
+					ProviderAddr: provider,
+					ActionTrigger: &plans.LifecycleActionTrigger{
+						ActionTriggerEvent:      configs.BeforeCreate,
+						ActionTriggerBlockIndex: 2,
+						ActionsListIndex:        1,
+						TriggeringResourceAddr: addrs.Resource{
+							Mode: addrs.ManagedResourceMode,
+							Type: "test_thing",
+							Name: "woot",
+						}.Instance(addrs.IntKey(0)).Absolute(addrs.RootModuleInstance),
+					},
+					ConfigValue: mustNewDynamicValue(cty.ObjectVal(map[string]cty.Value{
+						"id": cty.StringVal("secret"),
+					}), objTy),
+					SensitiveConfigPaths: []cty.Path{cty.GetAttrPath("id")},
+				},
+			},
 		},
 		DriftedResources: []*plans.ResourceInstanceChangeSrc{
 			{
@@ -316,10 +367,7 @@ func examplePlanForTest(t *testing.T) *plans.Plan {
 					Type: "test_thing",
 					Name: "woot",
 				}.Instance(addrs.NoKey).Absolute(addrs.RootModuleInstance),
-				ProviderAddr: addrs.AbsProviderConfig{
-					Provider: addrs.NewDefaultProvider("test"),
-					Module:   addrs.RootModule,
-				},
+				ProviderAddr: provider,
 				ChangeSrc: plans.ChangeSrc{
 					Action: plans.DeleteThenCreate,
 					Before: mustNewDynamicValue(cty.ObjectVal(map[string]cty.Value{
@@ -350,10 +398,7 @@ func examplePlanForTest(t *testing.T) *plans.Plan {
 						Type: "test_thing",
 						Name: "woot",
 					}.Instance(addrs.WildcardKey).Absolute(addrs.RootModuleInstance),
-					ProviderAddr: addrs.AbsProviderConfig{
-						Provider: addrs.NewDefaultProvider("test"),
-						Module:   addrs.RootModule,
-					},
+					ProviderAddr: provider,
 					ChangeSrc: plans.ChangeSrc{
 						Action: plans.Create,
 						After: mustNewDynamicValue(cty.ObjectVal(map[string]cty.Value{
@@ -379,10 +424,7 @@ func examplePlanForTest(t *testing.T) *plans.Plan {
 							InstanceKey: addrs.WildcardKey,
 						},
 					}),
-					ProviderAddr: addrs.AbsProviderConfig{
-						Provider: addrs.NewDefaultProvider("test"),
-						Module:   addrs.RootModule,
-					},
+					ProviderAddr: provider,
 					ChangeSrc: plans.ChangeSrc{
 						Action: plans.Create,
 						After: mustNewDynamicValue(cty.ObjectVal(map[string]cty.Value{
@@ -396,6 +438,30 @@ func examplePlanForTest(t *testing.T) *plans.Plan {
 				},
 			},
 		},
+		DeferredActionInvocations: []*plans.DeferredActionInvocationSrc{
+			{
+				DeferredReason: providers.DeferredReasonDeferredPrereq,
+				ActionInvocationInstanceSrc: &plans.ActionInvocationInstanceSrc{
+					Addr: addrs.Action{Type: "test_action", Name: "example"}.Absolute(addrs.RootModuleInstance).Instance(addrs.NoKey),
+					ActionTrigger: &plans.LifecycleActionTrigger{
+						TriggeringResourceAddr: addrs.Resource{
+							Mode: addrs.ManagedResourceMode,
+							Type: "test_thing",
+							Name: "woot",
+						}.Instance(addrs.IntKey(0)).Absolute(addrs.RootModuleInstance),
+						ActionTriggerBlockIndex: 1,
+						ActionsListIndex:        2,
+						ActionTriggerEvent:      configs.AfterCreate,
+					},
+					ProviderAddr: provider,
+					ConfigValue: mustNewDynamicValue(cty.ObjectVal(map[string]cty.Value{
+						"attr": cty.StringVal("value"),
+					}), cty.Object(map[string]cty.Type{
+						"attr": cty.String,
+					})),
+				},
+			},
+		},
 		RelevantAttributes: []globalref.ResourceAttr{
 			{
 				Resource: addrs.Resource{
