@@ -13,21 +13,21 @@ import (
 
 	"github.com/google/go-cmp/cmp"
 	"github.com/google/go-cmp/cmp/cmpopts"
-	"github.com/hashicorp/terraform/internal/addrs"
-	"github.com/hashicorp/terraform/internal/configs/configschema"
-	"github.com/hashicorp/terraform/internal/configs/hcl2shim"
-	"github.com/hashicorp/terraform/internal/plans"
-	"github.com/hashicorp/terraform/internal/providers"
-	"github.com/hashicorp/terraform/internal/schemarepo"
-	"github.com/hashicorp/terraform/internal/tfdiags"
 	"github.com/zclconf/go-cty/cty"
 	"go.uber.org/mock/gomock"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
 	"google.golang.org/protobuf/types/known/timestamppb"
 
+	"github.com/hashicorp/terraform/internal/addrs"
+	"github.com/hashicorp/terraform/internal/configs/configschema"
+	"github.com/hashicorp/terraform/internal/configs/hcl2shim"
+	"github.com/hashicorp/terraform/internal/plans"
 	"github.com/hashicorp/terraform/internal/plugin6/convert"
 	mockproto "github.com/hashicorp/terraform/internal/plugin6/mock_proto"
+	"github.com/hashicorp/terraform/internal/providers"
+	"github.com/hashicorp/terraform/internal/schemarepo"
+	"github.com/hashicorp/terraform/internal/tfdiags"
 	proto "github.com/hashicorp/terraform/internal/tfplugin6"
 )
 
@@ -156,6 +156,34 @@ func providerProtoSchema() *proto.GetProviderSchema_Response {
 				},
 			},
 		},
+		ActionSchemas: map[string]*proto.ActionSchema{
+			"action": {
+				Schema: &proto.Schema{
+					Block: &proto.Schema_Block{
+						Version: 1,
+						Attributes: []*proto.Schema_Attribute{
+							{
+								Name: "attr",
+								Type: []byte(`"string"`),
+							},
+						},
+					},
+				},
+			},
+		},
+		StateStoreSchemas: map[string]*proto.Schema{
+			"mock_store": {
+				Block: &proto.Schema_Block{
+					Version: 1,
+					Attributes: []*proto.Schema_Attribute{
+						{
+							Name: "region",
+							Type: []byte(`"string"`),
+						},
+					},
+				},
+			},
+		},
 		ServerCapabilities: &proto.ServerCapabilities{
 			GetProviderSchemaOptional: true,
 		},
@@ -494,10 +522,15 @@ func TestGRPCProvider_ValidateListResourceConfig_OptionalCfg(t *testing.T) {
 		gomock.Any(),
 	).Return(&proto.ValidateListResourceConfig_Response{}, nil)
 
+	converted := convert.ProtoToListSchema(sch.ListResourceSchemas["list"])
 	cfg := hcl2shim.HCL2ValueFromConfigValue(map[string]interface{}{})
+	coercedCfg, err := converted.Body.CoerceValue(cfg)
+	if err != nil {
+		t.Fatalf("failed to coerce config: %v", err)
+	}
 	resp := p.ValidateListResourceConfig(providers.ValidateListResourceConfigRequest{
 		TypeName: "list",
-		Config:   cfg,
+		Config:   coercedCfg,
 	})
 	checkDiags(t, resp.Diagnostics)
 }
@@ -1430,7 +1463,9 @@ func TestGRPCProvider_GetSchema_ListResourceTypes(t *testing.T) {
 								},
 							},
 						},
-						Nesting: configschema.NestingSingle,
+						Nesting:  configschema.NestingSingle,
+						MinItems: 1,
+						MaxItems: 1,
 					},
 				},
 			},
@@ -1668,39 +1703,6 @@ func TestGRPCProvider_ListResource_Error(t *testing.T) {
 }
 
 func TestGRPCProvider_ListResource_Diagnostics(t *testing.T) {
-	client := mockProviderClient(t)
-	p := &GRPCProvider{
-		client: client,
-		ctx:    context.Background(),
-	}
-
-	// Create a mock stream client that will return a resource event with diagnostics
-	mockStream := &mockListResourceStreamClient{
-		events: []*proto.ListResource_Event{
-			{
-				DisplayName: "Test Resource With Warning",
-				Identity: &proto.ResourceIdentityData{
-					IdentityData: &proto.DynamicValue{
-						Msgpack: []byte("\x81\xa7id_attr\xa4id-1"),
-					},
-				},
-				Diagnostic: []*proto.Diagnostic{
-					{
-						Severity: proto.Diagnostic_WARNING,
-						Summary:  "Test warning",
-						Detail:   "This is a test warning",
-					},
-				},
-			},
-		},
-	}
-
-	client.EXPECT().ListResource(
-		gomock.Any(),
-		gomock.Any(),
-	).Return(mockStream, nil)
-
-	// Create the request
 	configVal := cty.ObjectVal(map[string]cty.Value{
 		"config": cty.ObjectVal(map[string]cty.Value{
 			"filter_attr": cty.StringVal("filter-value"),
@@ -1712,16 +1714,212 @@ func TestGRPCProvider_ListResource_Diagnostics(t *testing.T) {
 		Limit:    100,
 	}
 
-	resp := p.ListResource(request)
-	checkDiags(t, resp.Diagnostics)
-
-	data := resp.Result.AsValueMap()
-	if _, ok := data["data"]; !ok {
-		t.Fatal("Expected 'data' key in result")
+	testCases := []struct {
+		name          string
+		events        []*proto.ListResource_Event
+		expectedCount int
+		expectedDiags int
+		expectedWarns int // subset of expectedDiags
+	}{
+		{
+			"no events",
+			[]*proto.ListResource_Event{},
+			0,
+			0,
+			0,
+		},
+		{
+			"single event no diagnostics",
+			[]*proto.ListResource_Event{
+				{
+					DisplayName: "Test Resource",
+					Identity: &proto.ResourceIdentityData{
+						IdentityData: &proto.DynamicValue{
+							Msgpack: []byte("\x81\xa7id_attr\xa4id-1"),
+						},
+					},
+				},
+			},
+			1,
+			0,
+			0,
+		},
+		{
+			"event with warning",
+			[]*proto.ListResource_Event{
+				{
+					DisplayName: "Test Resource",
+					Identity: &proto.ResourceIdentityData{
+						IdentityData: &proto.DynamicValue{
+							Msgpack: []byte("\x81\xa7id_attr\xa4id-1"),
+						},
+					},
+					Diagnostic: []*proto.Diagnostic{
+						{
+							Severity: proto.Diagnostic_WARNING,
+							Summary:  "Test warning",
+							Detail:   "Warning detail",
+						},
+					},
+				},
+			},
+			1,
+			1,
+			1,
+		},
+		{
+			"only a warning",
+			[]*proto.ListResource_Event{
+				{
+					Diagnostic: []*proto.Diagnostic{
+						{
+							Severity: proto.Diagnostic_WARNING,
+							Summary:  "Test warning",
+							Detail:   "Warning detail",
+						},
+					},
+				},
+			},
+			0,
+			1,
+			1,
+		},
+		{
+			"only an error",
+			[]*proto.ListResource_Event{
+				{
+					Diagnostic: []*proto.Diagnostic{
+						{
+							Severity: proto.Diagnostic_ERROR,
+							Summary:  "Test error",
+							Detail:   "Error detail",
+						},
+					},
+				},
+			},
+			0,
+			1,
+			0,
+		},
+		{
+			"event with error",
+			[]*proto.ListResource_Event{
+				{
+					DisplayName: "Test Resource",
+					Identity: &proto.ResourceIdentityData{
+						IdentityData: &proto.DynamicValue{
+							Msgpack: []byte("\x81\xa7id_attr\xa4id-1"),
+						},
+					},
+					Diagnostic: []*proto.Diagnostic{
+						{
+							Severity: proto.Diagnostic_ERROR,
+							Summary:  "Test error",
+							Detail:   "Error detail",
+						},
+					},
+				},
+			},
+			0,
+			1,
+			0,
+		},
+		{
+			"multiple events mixed diagnostics",
+			[]*proto.ListResource_Event{
+				{
+					DisplayName: "Resource 1",
+					Identity: &proto.ResourceIdentityData{
+						IdentityData: &proto.DynamicValue{
+							Msgpack: []byte("\x81\xa7id_attr\xa4id-1"),
+						},
+					},
+					Diagnostic: []*proto.Diagnostic{
+						{
+							Severity: proto.Diagnostic_WARNING,
+							Summary:  "Warning 1",
+							Detail:   "Warning detail 1",
+						},
+					},
+				},
+				{
+					DisplayName: "Resource 2",
+					Identity: &proto.ResourceIdentityData{
+						IdentityData: &proto.DynamicValue{
+							Msgpack: []byte("\x81\xa7id_attr\xa4id-2"),
+						},
+					},
+				},
+				{
+					DisplayName: "Resource 3",
+					Identity: &proto.ResourceIdentityData{
+						IdentityData: &proto.DynamicValue{
+							Msgpack: []byte("\x81\xa7id_attr\xa4id-3"),
+						},
+					},
+					Diagnostic: []*proto.Diagnostic{
+						{
+							Severity: proto.Diagnostic_ERROR,
+							Summary:  "Error 1",
+							Detail:   "Error detail 1",
+						},
+						{
+							Severity: proto.Diagnostic_WARNING,
+							Summary:  "Warning 2",
+							Detail:   "Warning detail 2",
+						},
+					},
+				},
+				{ // This event will never be reached
+					DisplayName: "Resource 4",
+					Identity: &proto.ResourceIdentityData{
+						IdentityData: &proto.DynamicValue{
+							Msgpack: []byte("\x81\xa7id_attr\xa4id-4"),
+						},
+					},
+				},
+			},
+			2,
+			3,
+			2,
+		},
 	}
 
-	if !resp.Diagnostics.HasWarnings() {
-		t.Fatal("Expected warning diagnostics, but got none")
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			client := mockProviderClient(t)
+			p := &GRPCProvider{
+				client: client,
+				ctx:    context.Background(),
+			}
+
+			mockStream := &mockListResourceStreamClient{
+				events: tc.events,
+			}
+
+			client.EXPECT().ListResource(
+				gomock.Any(),
+				gomock.Any(),
+			).Return(mockStream, nil)
+
+			resp := p.ListResource(request)
+
+			result := resp.Result.AsValueMap()
+			nResults := result["data"].LengthInt()
+			if nResults != tc.expectedCount {
+				t.Fatalf("Expected %d results, got %d", tc.expectedCount, nResults)
+			}
+
+			nDiagnostics := len(resp.Diagnostics)
+			if nDiagnostics != tc.expectedDiags {
+				t.Fatalf("Expected %d diagnostics, got %d", tc.expectedDiags, nDiagnostics)
+			}
+
+			nWarnings := len(resp.Diagnostics.Warnings())
+			if nWarnings != tc.expectedWarns {
+				t.Fatalf("Expected %d warnings, got %d", tc.expectedWarns, nWarnings)
+			}
+		})
 	}
 }
 
@@ -1797,3 +1995,597 @@ func TestGRPCProvider_ListResource_Limit(t *testing.T) {
 		t.Fatalf("Expected 2 events, got %d", len(results))
 	}
 }
+
+func TestGRPCProvider_planAction_valid(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+		ctx:    context.Background(),
+	}
+
+	client.EXPECT().PlanAction(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(&proto.PlanAction_Response{}, nil)
+
+	resp := p.PlanAction(providers.PlanActionRequest{
+		ActionType: "action",
+		ProposedActionData: cty.ObjectVal(map[string]cty.Value{
+			"attr": cty.StringVal("foo"),
+		}),
+	})
+
+	checkDiags(t, resp.Diagnostics)
+}
+
+func TestGRPCProvider_planAction_valid_but_fails(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+	}
+
+	client.EXPECT().PlanAction(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(&proto.PlanAction_Response{
+		Diagnostics: []*proto.Diagnostic{
+			{
+				Severity: proto.Diagnostic_ERROR,
+				Summary:  "Boom",
+				Detail:   "Explosion",
+			},
+		},
+	}, nil)
+
+	resp := p.PlanAction(providers.PlanActionRequest{
+		ActionType: "action",
+		ProposedActionData: cty.ObjectVal(map[string]cty.Value{
+			"attr": cty.StringVal("foo"),
+		}),
+	})
+
+	checkDiagsHasError(t, resp.Diagnostics)
+}
+
+func TestGRPCProvider_planAction_invalid_config(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+	}
+
+	resp := p.PlanAction(providers.PlanActionRequest{
+		ActionType: "action",
+		ProposedActionData: cty.ObjectVal(map[string]cty.Value{
+			"not_the_right_attr": cty.StringVal("foo"),
+		}),
+	})
+
+	checkDiagsHasError(t, resp.Diagnostics)
+}
+
+func TestGRPCProvider_invokeAction_valid(t *testing.T) {
+	ctrl := gomock.NewController(t)
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+	}
+
+	mockInvokeClient := mockproto.NewMockProvider_InvokeActionClient(ctrl)
+	mockInvokeClient.EXPECT().Recv().Return(&proto.InvokeAction_Event{
+		Type: &proto.InvokeAction_Event_Progress_{
+			Progress: &proto.InvokeAction_Event_Progress{
+				Message: "Hello from the action",
+			},
+		},
+	}, nil)
+	mockInvokeClient.EXPECT().Recv().Return(&proto.InvokeAction_Event{
+		Type: &proto.InvokeAction_Event_Completed_{
+			Completed: &proto.InvokeAction_Event_Completed{},
+		},
+	}, nil)
+	mockInvokeClient.EXPECT().Recv().Return(nil, io.EOF)
+
+	client.EXPECT().InvokeAction(
+		gomock.Any(),
+		gomock.Any(),
+	).Return(mockInvokeClient, nil)
+
+	resp := p.InvokeAction(providers.InvokeActionRequest{
+		ActionType: "action",
+		PlannedActionData: cty.ObjectVal(map[string]cty.Value{
+			"attr": cty.StringVal("foo"),
+		}),
+	})
+
+	evts := []providers.InvokeActionEvent{}
+	for e := range resp.Events {
+		evts = append(evts, e)
+	}
+
+	if len(evts) != 2 {
+		t.Fatalf("expected 2 events, got %d", len(evts))
+	}
+
+	checkDiags(t, resp.Diagnostics)
+}
+
+func TestGRPCProvider_invokeAction_invalid(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+	}
+
+	resp := p.InvokeAction(providers.InvokeActionRequest{
+		ActionType: "action",
+		PlannedActionData: cty.ObjectVal(map[string]cty.Value{
+			"not-defined": cty.StringVal("foo"),
+		}),
+	})
+
+	checkDiagsHasError(t, resp.Diagnostics)
+}
+
+func TestGRPCProvider_ValidateStateStoreConfig_returns_validation_errors(t *testing.T) {
+	storeName := "mock_store" // mockProviderClient returns a mock that has this state store in its schemas
+
+	t.Run("no validation error raised", func(t *testing.T) {
+		typeName := storeName
+		var diagnostic []*proto.Diagnostic = nil
+
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		client.EXPECT().ValidateStateStoreConfig(
+			gomock.Any(),
+			gomock.Any(),
+		).Return(&proto.ValidateStateStore_Response{
+			Diagnostics: diagnostic,
+		}, nil)
+
+		request := providers.ValidateStateStoreConfigRequest{
+			TypeName: typeName,
+			Config: cty.ObjectVal(map[string]cty.Value{
+				"region": cty.StringVal("neptune"),
+			}),
+		}
+
+		// Act
+		resp := p.ValidateStateStoreConfig(request)
+
+		// Assert no error returned
+		checkDiags(t, resp.Diagnostics)
+
+	})
+
+	t.Run("validation error raised", func(t *testing.T) {
+		typeName := storeName
+		diagnostic := []*proto.Diagnostic{
+			{
+				Severity: proto.Diagnostic_ERROR,
+				Summary:  "Error from ValidateStateStoreConfig",
+				Detail:   "Something went wrong",
+			},
+		}
+		errorText := "Error from ValidateStateStoreConfig"
+
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		client.EXPECT().ValidateStateStoreConfig(
+			gomock.Any(),
+			gomock.Any(),
+		).Return(&proto.ValidateStateStore_Response{
+			Diagnostics: diagnostic,
+		}, nil)
+
+		request := providers.ValidateStateStoreConfigRequest{
+			TypeName: typeName,
+			Config: cty.ObjectVal(map[string]cty.Value{
+				"region": cty.StringVal("neptune"),
+			}),
+		}
+
+		// Act
+		resp := p.ValidateStateStoreConfig(request)
+
+		// Assert error returned
+		checkDiagsHasError(t, resp.Diagnostics)
+		if resp.Diagnostics[0].Description().Summary != errorText {
+			t.Fatalf("expected error summary to be %q, but got %q",
+				errorText,
+				resp.Diagnostics[0].Description().Summary,
+			)
+		}
+
+	})
+
+}
+
+func TestGRPCProvider_ValidateStateStoreConfig_schema_errors(t *testing.T) {
+	t.Run("no matching store type in provider", func(t *testing.T) {
+		typeName := "does_not_exist" // not present in mockProviderClient state store schemas
+		config := cty.EmptyObjectVal
+		expectedErrorSummary := "unknown state store type \"does_not_exist\""
+
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		request := providers.ValidateStateStoreConfigRequest{
+			TypeName: typeName,
+			Config:   config,
+		}
+
+		// Act
+		resp := p.ValidateStateStoreConfig(request)
+
+		// Note - we haven't asserted that we expect ValidateStateStoreConfig
+		// to be called via the client; this package returns these errors before then.
+
+		// Assert that the expected error is returned
+		checkDiagsHasError(t, resp.Diagnostics)
+		if resp.Diagnostics[0].Description().Summary != expectedErrorSummary {
+			t.Fatalf("expected error summary to be %q, but got %q",
+				expectedErrorSummary,
+				resp.Diagnostics[0].Description().Summary,
+			)
+		}
+	})
+
+	t.Run("missing required attributes", func(t *testing.T) {
+		typeName := "mock_store" // Is present in mockProviderClient
+		config := cty.ObjectVal(map[string]cty.Value{
+			// Missing required `region` attr
+		})
+		expectedErrorSummary := "attribute \"region\" is required"
+
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		request := providers.ValidateStateStoreConfigRequest{
+			TypeName: typeName,
+			Config:   config,
+		}
+
+		// Act
+		resp := p.ValidateStateStoreConfig(request)
+
+		// Note - we haven't asserted that we expect ValidateStateStoreConfig
+		// to be called via the client; this package returns these errors before then.
+
+		// Assert that the expected error is returned
+		checkDiagsHasError(t, resp.Diagnostics)
+		if resp.Diagnostics[0].Description().Summary != expectedErrorSummary {
+			t.Fatalf("expected error summary to be %q, but got %q",
+				expectedErrorSummary,
+				resp.Diagnostics[0].Description().Summary,
+			)
+		}
+	})
+}
+
+func TestGRPCProvider_ConfigureStateStore_returns_validation_errors(t *testing.T) {
+	storeName := "mock_store" // mockProviderClient returns a mock that has this state store in its schemas
+
+	t.Run("no validation error raised", func(t *testing.T) {
+		typeName := storeName
+		var diagnostic []*proto.Diagnostic = nil
+
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		client.EXPECT().ConfigureStateStore(
+			gomock.Any(),
+			gomock.Any(),
+		).Return(&proto.ConfigureStateStore_Response{
+			Diagnostics: diagnostic,
+		}, nil)
+
+		request := providers.ConfigureStateStoreRequest{
+			TypeName: typeName,
+			Config: cty.ObjectVal(map[string]cty.Value{
+				"region": cty.StringVal("neptune"),
+			}),
+		}
+
+		// Act
+		resp := p.ConfigureStateStore(request)
+
+		// Assert no error returned
+		checkDiags(t, resp.Diagnostics)
+	})
+
+	t.Run("validation error raised", func(t *testing.T) {
+		typeName := storeName
+		diagnostic := []*proto.Diagnostic{
+			{
+				Severity: proto.Diagnostic_ERROR,
+				Summary:  "Error from ConfigureStateStore",
+				Detail:   "Something went wrong",
+			},
+		}
+		errorText := "Error from ConfigureStateStore"
+
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		client.EXPECT().ConfigureStateStore(
+			gomock.Any(),
+			gomock.Any(),
+		).Return(&proto.ConfigureStateStore_Response{
+			Diagnostics: diagnostic,
+		}, nil)
+
+		request := providers.ConfigureStateStoreRequest{
+			TypeName: typeName,
+			Config: cty.ObjectVal(map[string]cty.Value{
+				"region": cty.StringVal("neptune"),
+			}),
+		}
+
+		// Act
+		resp := p.ConfigureStateStore(request)
+
+		// Assert whether error returned or not
+		checkDiagsHasError(t, resp.Diagnostics)
+		if resp.Diagnostics[0].Description().Summary != errorText {
+			t.Fatalf("expected error summary to be %q, but got %q",
+				errorText,
+				resp.Diagnostics[0].Description().Summary,
+			)
+		}
+
+	})
+
+}
+
+func TestGRPCProvider_ConfigureStateStore_schema_errors(t *testing.T) {
+	t.Run("no matching store type in provider", func(t *testing.T) {
+		typeName := "does_not_exist" // not present in mockProviderClient state store schemas
+		config := cty.EmptyObjectVal
+		expectedErrorSummary := "unknown state store type \"does_not_exist\""
+
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		request := providers.ConfigureStateStoreRequest{
+			TypeName: typeName,
+			Config:   config,
+		}
+
+		// Act
+		resp := p.ConfigureStateStore(request)
+
+		// Note - we haven't asserted that we expect ConfigureStateStore
+		// to be called via the client; this package returns these errors before then.
+
+		// Assert that the expected error is returned
+		checkDiagsHasError(t, resp.Diagnostics)
+		if resp.Diagnostics[0].Description().Summary != expectedErrorSummary {
+			t.Fatalf("expected error summary to be %q, but got %q",
+				expectedErrorSummary,
+				resp.Diagnostics[0].Description().Summary,
+			)
+		}
+	})
+
+	t.Run("missing required attributes", func(t *testing.T) {
+		typeName := "mock_store" // Is present in mockProviderClient
+		config := cty.ObjectVal(map[string]cty.Value{
+			// Missing required `region` attr
+		})
+		expectedErrorSummary := "attribute \"region\" is required"
+
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		request := providers.ConfigureStateStoreRequest{
+			TypeName: typeName,
+			Config:   config,
+		}
+
+		// Act
+		resp := p.ConfigureStateStore(request)
+
+		// Note - we haven't asserted that we expect ConfigureStateStore
+		// to be called via the client; this package returns these errors before then.
+
+		// Assert that the expected error is returned
+		checkDiagsHasError(t, resp.Diagnostics)
+		if resp.Diagnostics[0].Description().Summary != expectedErrorSummary {
+			t.Fatalf("expected error summary to be %q, but got %q",
+				expectedErrorSummary,
+				resp.Diagnostics[0].Description().Summary,
+			)
+		}
+	})
+}
+
+func TestGRPCProvider_GetStates(t *testing.T) {
+	t.Run("returns expected values", func(t *testing.T) {
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		client.EXPECT().GetStates(
+			gomock.Any(),
+			gomock.Any(),
+		).Return(&proto.GetStates_Response{
+			StateId: []string{"default"},
+			Diagnostics: []*proto.Diagnostic{
+				{
+					Severity: proto.Diagnostic_ERROR,
+					Summary:  "Error from GetStates",
+					Detail:   "Something went wrong",
+				},
+			},
+		}, nil)
+
+		request := providers.GetStatesRequest{
+			TypeName: "mock_store",
+		}
+
+		// Act
+		resp := p.GetStates(request)
+
+		// Assert returned values
+		if len(resp.States) != 1 || resp.States[0] != "default" {
+			t.Fatalf("expected the returned states to be [\"default\"], instead got: %s", resp.States)
+		}
+
+		checkDiagsHasError(t, resp.Diagnostics)
+		expectedErrorSummary := "Error from GetStates"
+		if resp.Diagnostics[0].Description().Summary != expectedErrorSummary {
+			t.Fatalf("expected error summary to be %q, but got %q",
+				expectedErrorSummary,
+				resp.Diagnostics[0].Description().Summary,
+			)
+		}
+	})
+
+	t.Run("no matching store type in provider", func(t *testing.T) {
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		request := providers.GetStatesRequest{
+			TypeName: "does_not_exist", // not present in mockProviderClient state store schemas
+		}
+
+		// Act
+		resp := p.GetStates(request)
+
+		checkDiagsHasError(t, resp.Diagnostics)
+		expectedErrorSummary := "unknown state store type \"does_not_exist\""
+		if resp.Diagnostics[0].Description().Summary != expectedErrorSummary {
+			t.Fatalf("expected error summary to be %q, but got %q",
+				expectedErrorSummary,
+				resp.Diagnostics[0].Description().Summary,
+			)
+		}
+	})
+}
+
+func TestGRPCProvider_DeleteState(t *testing.T) {
+	t.Run("returns expected values", func(t *testing.T) {
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		client.EXPECT().DeleteState(
+			gomock.Any(),
+			gomock.Any(),
+		).Return(&proto.DeleteState_Response{
+			Diagnostics: []*proto.Diagnostic{
+				{
+					Severity: proto.Diagnostic_ERROR,
+					Summary:  "Error from DeleteState",
+					Detail:   "Something went wrong",
+				},
+			},
+		}, nil)
+
+		request := providers.DeleteStateRequest{
+			TypeName: "mock_store",
+		}
+
+		// Act
+		resp := p.DeleteState(request)
+
+		// Assert returned values
+		checkDiagsHasError(t, resp.Diagnostics)
+		expectedErrorSummary := "Error from DeleteState"
+		if resp.Diagnostics[0].Description().Summary != expectedErrorSummary {
+			t.Fatalf("expected error summary to be %q, but got %q",
+				expectedErrorSummary,
+				resp.Diagnostics[0].Description().Summary,
+			)
+		}
+	})
+
+	t.Run("no matching store type in provider", func(t *testing.T) {
+		client := mockProviderClient(t)
+		p := &GRPCProvider{
+			client: client,
+			ctx:    context.Background(),
+		}
+
+		request := providers.DeleteStateRequest{
+			TypeName: "does_not_exist", // not present in mockProviderClient state store schemas
+		}
+
+		// Act
+		resp := p.DeleteState(request)
+
+		checkDiagsHasError(t, resp.Diagnostics)
+		expectedErrorSummary := "unknown state store type \"does_not_exist\""
+		if resp.Diagnostics[0].Description().Summary != expectedErrorSummary {
+			t.Fatalf("expected error summary to be %q, but got %q",
+				expectedErrorSummary,
+				resp.Diagnostics[0].Description().Summary,
+			)
+		}
+	})
+}
+
+func TestGRPCProvider_GenerateResourceConfig(t *testing.T) {
+	client := mockProviderClient(t)
+	p := &GRPCProvider{
+		client: client,
+	}
+	client.EXPECT().GenerateResourceConfig(
+		gomock.Any(),
+		gomock.Cond[any](func(x any) bool {
+			req := x.(*proto.GenerateResourceConfig_Request)
+			if req.TypeName != "resource" {
+				return false
+			}
+			if req.State == nil {
+				t.Log("GenerateResourceConfig state is nil")
+				return false
+			}
+			return true
+		}),
+	).Return(&proto.GenerateResourceConfig_Response{
+		Config: &proto.DynamicValue{
+			Msgpack: []byte("\x81\xa4attr\xa3bar"),
+		},
+	}, nil)
+	resp := p.GenerateResourceConfig(providers.GenerateResourceConfigRequest{
+		TypeName: "resource",
+		State: cty.ObjectVal(map[string]cty.Value{
+			"computed": cty.StringVal("computed"),
+			"attr":     cty.StringVal("foo"),
+		}),
+	})
+	checkDiags(t, resp.Diagnostics)
+}
