@@ -37,3 +37,30 @@ func (ss ProviderSchema) SchemaForResourceAddr(addr addrs.Resource) (schema Sche
 }
 
 type ResourceIdentitySchemas = GetResourceIdentitySchemasResponse
+
+// SchemaForActionType attempts to find a schema for the given type. Returns an
+// empty schema if none is available.
+func (ss ProviderSchema) SchemaForActionType(typeName string) (schema ActionSchema) {
+	schema, ok := ss.Actions[typeName]
+	if ok {
+		return schema
+	}
+	return ActionSchema{}
+}
+
+// SchemaForListResourceType attempts to find a schema for the given type. Returns an
+// empty schema if none is available.
+func (ss ProviderSchema) SchemaForListResourceType(typeName string) ListResourceSchema {
+	schema, ok := ss.ListResourceTypes[typeName]
+	ret := ListResourceSchema{FullSchema: schema.Body}
+	if !ok || schema.Body == nil {
+		return ret
+	}
+	// The configuration for the list block is nested within a "config" block.
+	configSchema, ok := schema.Body.BlockTypes["config"]
+	if !ok {
+		return ret
+	}
+	ret.ConfigSchema = &configSchema.Block
+	return ret
+}
