@@ -34,6 +34,16 @@ type ValidateOpts struct {
 	// not available to this function. Therefore, it is the responsibility of
 	// the caller to ensure that the provider configurations are valid.
 	ExternalProviders map[addrs.RootProviderConfig]providers.Interface
+
+	// When true, query files will also be validated.
+	Query bool
+
+	// AllowRootEphemeralOutputs overrides a specific check made within the
+	// output nodes that they cannot be ephemeral at within root modules. This
+	// should be set to true for plans executing from within either the stacks
+	// or test runtimes, where the root modules as Terraform sees them aren't
+	// the actual root modules.
+	AllowRootEphemeralOutputs bool
 }
 
 // Validate performs semantic validation of a configuration, and returns
@@ -98,13 +108,15 @@ func (c *Context) Validate(config *configs.Config, opts *ValidateOpts) tfdiags.D
 	}
 
 	graph, moreDiags := (&PlanGraphBuilder{
-		Config:                  config,
-		Plugins:                 c.plugins,
-		State:                   states.NewState(),
-		RootVariableValues:      varValues,
-		Operation:               walkValidate,
-		ExternalProviderConfigs: opts.ExternalProviders,
-		ImportTargets:           c.findImportTargets(config),
+		Config:                    config,
+		Plugins:                   c.plugins,
+		State:                     states.NewState(),
+		RootVariableValues:        varValues,
+		Operation:                 walkValidate,
+		ExternalProviderConfigs:   opts.ExternalProviders,
+		ImportTargets:             c.findImportTargets(config),
+		queryPlan:                 opts.Query,
+		AllowRootEphemeralOutputs: opts.AllowRootEphemeralOutputs,
 	}).Build(addrs.RootModuleInstance)
 	diags = diags.Append(moreDiags)
 	if moreDiags.HasErrors() {
