@@ -13,6 +13,7 @@ import (
 	"github.com/zclconf/go-cty/cty"
 	"github.com/zclconf/go-cty/cty/function"
 
+	"github.com/hashicorp/terraform/internal/actions"
 	"github.com/hashicorp/terraform/internal/addrs"
 	"github.com/hashicorp/terraform/internal/checks"
 	"github.com/hashicorp/terraform/internal/configs"
@@ -91,6 +92,7 @@ type BuiltinEvalContext struct {
 	InstanceExpanderValue   *instances.Expander
 	MoveResultsValue        refactoring.MoveResults
 	OverrideValues          *mocking.Overrides
+	ActionsValue            *actions.Actions
 }
 
 // BuiltinEvalContext implements EvalContext
@@ -637,3 +639,7 @@ func (ctx *BuiltinEvalContext) ClientCapabilities() providers.ClientCapabilities
 		WriteOnlyAttributesAllowed: true,
 	}
 }
+
+func (ctx *BuiltinEvalContext) Actions() *actions.Actions {
+	return ctx.ActionsValue
+}
