@@ -89,7 +89,7 @@ func testModuleWithSnapshot(t *testing.T, name string) (*configs.Config, *config
 
 // testModuleInline takes a map of path -> config strings and yields a config
 // structure with those files loaded from disk
-func testModuleInline(t testing.TB, sources map[string]string) *configs.Config {
+func testModuleInline(t testing.TB, sources map[string]string, parserOpts ...configs.Option) *configs.Config {
 	t.Helper()
 
 	cfgPath, err := filepath.EvalSymlinks(t.TempDir())
@@ -118,7 +118,7 @@ func testModuleInline(t testing.TB, sources map[string]string) *configs.Config {
 		}
 	}
 
-	loader, cleanup := configload.NewLoaderForTests(t)
+	loader, cleanup := configload.NewLoaderForTests(t, parserOpts...)
 	defer cleanup()
 
 	// We need to be able to exercise experimental features in our integration tests.
@@ -139,7 +139,7 @@ func testModuleInline(t testing.TB, sources map[string]string) *configs.Config {
 		t.Fatalf("failed to refresh modules after installation: %s", err)
 	}
 
-	config, diags := loader.LoadConfig(cfgPath, configs.MatchTestFiles("tests"))
+	config, diags := loader.LoadConfigWithTests(cfgPath, "tests")
 	if diags.HasErrors() {
 		t.Fatal(diags.Error())
 	}
