@@ -42,6 +42,15 @@ type GraphNodeAttachProvisionerSchema interface {
 	AttachProvisionerSchema(name string, schema *configschema.Block)
 }
 
+// GraphNodeAttachActionSchema is an interface implemented by node types
+// that need a resource schema attached.
+type GraphNodeAttachActionSchema interface {
+	GraphNodeConfigAction
+	GraphNodeProviderConsumer
+
+	AttachActionSchema(schema *providers.ActionSchema)
+}
+
 // AttachSchemaTransformer finds nodes that implement
 // GraphNodeAttachResourceSchema, GraphNodeAttachProviderConfigSchema, or
 // GraphNodeAttachProvisionerSchema, looks up the needed schemas for each
@@ -107,6 +116,21 @@ func (t *AttachSchemaTransformer) Transform(g *Graph) error {
 				tv.AttachProvisionerSchema(name, schema)
 			}
 		}
+
+		if tv, ok := v.(GraphNodeAttachActionSchema); ok {
+			addr := tv.ActionAddr()
+			providerFqn := tv.Provider()
+			schema, err := t.Plugins.ActionTypeSchema(providerFqn, addr.Action.Type)
+			if err != nil {
+				return fmt.Errorf("failed to read schema for %s in %s: %s", addr, providerFqn, err)
+			}
+			if schema.ConfigSchema == nil {
+				log.Printf("[ERROR] AttachSchemaTransformer: No action schema available for %s", addr)
+				continue
+			}
+			log.Printf("[TRACE] AttachSchemaTransformer: attaching action schema to %s", dag.VertexName(v))
+			tv.AttachActionSchema(schema)
+		}
 	}
 
 	return nil
